@@ -32,11 +32,11 @@ import socket
 from typing import Optional
 
 from startrek.types import Address
-from startrek.net.channel import is_opened, is_bound, is_connected
 from startrek import BaseChannel, ChannelReader, ChannelWriter
 
 
-class PackageChannelReader(ChannelReader):
+class PacketChannelReader(ChannelReader):
+    """ Datagram Packet Channel Reader """
 
     def _try_receive(self, max_len: int, sock: socket.socket) -> (Optional[bytes], Optional[Address]):
         try:
@@ -49,8 +49,7 @@ class PackageChannelReader(ChannelReader):
             # received nothing
             return None, None
 
-    def _receive_from(self, max_len: int) -> (Optional[bytes], Optional[Address]):
-        sock = self.sock
+    def _receive_from(self, max_len: int, sock: socket.socket) -> (Optional[bytes], Optional[Address]):
         data, remote = self._try_receive(max_len=max_len, sock=sock)
         # check data
         error = self._check_data(data=data, sock=sock)
@@ -65,13 +64,14 @@ class PackageChannelReader(ChannelReader):
         remote = self.remote_address
         if remote is None:
             # not connect (UDP)
-            return self._receive_from(max_len=max_len)
+            return self._receive_from(max_len=max_len, sock=self.sock)
         else:
             # connected (TCP/UDP)
             return self.read(max_len=max_len), remote
 
 
-class PackageChannelWriter(ChannelWriter):
+class PacketChannelWriter(ChannelWriter):
+    """ Datagram Packet Channel Writer """
 
     def _try_send(self, data: bytes, target: Address, sock: socket.socket) -> int:
         try:
@@ -90,68 +90,23 @@ class PackageChannelWriter(ChannelWriter):
         remote = self.remote_address
         if remote is None:
             # not connect (UDP)
+            assert target is not None, 'target missed for unbound channel'
             return self._try_send(data=data, target=target, sock=self.sock)
         else:
             # connected (TCP/UDP)
-            assert target is None or target == remote, 'target address error: %s, %s' % (target, remote)
+            remote = self.remote_address
+            assert target is None or target == remote, 'target error: %s, remote=%s' % (target, remote)
             # return sock.send(data)
             return self._try_write(data=data, sock=self.sock)
 
 
-class PackageChannel(BaseChannel):
-    """ Discrete Package Channel """
-
-    def __init__(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket):
-        super().__init__(remote=remote, local=local)
-        self.__sock = sock
-        self._refresh_flags(sock=sock)
-
-    # Override
-    def _get_socket(self) -> Optional[socket.socket]:
-        return self.__sock
-
-    # Override
-    def _set_socket(self, sock: Optional[socket.socket]):
-        # 1. replace with new socket
-        old = self.__sock
-        self.__sock = sock
-        # 2. refresh flags with new socket
-        self._refresh_flags(sock=sock)
-        # 3. close old socket
-        if old is not None and old is not sock:
-            if is_opened(sock=old) and is_connected(sock=old):
-                # DON'T close bound socket
-                close_socket(sock=old)
-
-    # Override
-    def disconnect(self) -> Optional[socket.socket]:
-        # DON'T close bound socket channel
-        if self.bound and not self.connected:
-            return None
-        return super().disconnect()
-
-    # Override
-    def close(self):
-        # DON'T close bound socket channel
-        if self.bound and not self.connected:
-            return False
-        super().close()
+class PacketChannel(BaseChannel):
+    """ Datagram Packet Channel """
 
     # Override
     def _create_reader(self):
-        return PackageChannelReader(channel=self)
+        return PacketChannelReader(channel=self)
 
     # Override
     def _create_writer(self):
-        return PackageChannelWriter(channel=self)
-
-
-def close_socket(sock: socket.socket):
-    # DON'T close bound socket
-    if is_bound(sock=sock) and not is_connected(sock=sock):
-        return False
-    try:
-        # sock.shutdown(socket.SHUT_RDWR)
-        sock.close()
-    except socket.error as error:
-        print('[UDP] failed to close socket: %s, %s' % (error, sock))
+        return PacketChannelWriter(channel=self)
