@@ -38,11 +38,52 @@ from startrek import Connection, ConnectionDelegate
 from startrek import BaseConnection, ActiveConnection
 from startrek import BaseHub
 
-from .channel import PackageChannel
+from .channel import PacketChannel
 
 
 class ChannelPool(AddressPairMap[Channel]):
 
+    # Override
+    def get(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
+        assert not (remote is None and local is None), 'both addresses are empty'
+        # -- Step 1 --
+        #     (remote, local)
+        #         this step will get the channel connected to the remote address
+        #         and bound to the local address at the same time;
+        #     (null, local)
+        #         this step will get a channel bound to the local address, and
+        #         surely not connected;
+        #     (remote, null)
+        #         this step will get a channel connected to the remote address,
+        #         no matter which local address to be bound;
+        channel = super().get(remote=remote, local=local)
+        if channel is not None:
+            return channel
+        elif remote is None:
+            # failed to get a channel bound to the local address
+            return None
+        # -- Step 2 --
+        #     (remote, local)
+        #         try to get a channel which bound to the local address, but
+        #         not connected to any remote address;
+        #     (null, local)
+        #         this situation has already done at step 1;
+        if local is not None:
+            # ignore the remote address
+            channel = super().get(remote=None, local=local)
+            if isinstance(channel, Channel) and channel.remote_address is None:
+                # got a channel not connected yet
+                return channel
+        # -- Step 3 --
+        #     (remote, null)
+        #         try to get a channel that bound to any local address, but
+        #         not connected yet;
+        array = self.items
+        for item in array:
+            if item.remote_address is None:
+                return item
+        # not found
+
     # Override
     def set(self, remote: Optional[Address], local: Optional[Address], item: Optional[Channel]):
         old = self.get(remote=remote, local=local)
@@ -54,20 +95,25 @@ class ChannelPool(AddressPairMap[Channel]):
     def remove(self, remote: Optional[Address], local: Optional[Address], item: Optional[Channel]) -> Optional[Channel]:
         cached = super().remove(remote=remote, local=local, item=item)
         if cached is not None:
-            if cached.opened:
+            if not cached.closed:
                 cached.close()
             return cached
 
 
-class PackageHub(BaseHub, ABC):
-    """ Base Package Hub """
+class PacketHub(BaseHub, ABC):
+    """ Base Datagram Hub """
 
     def __init__(self, delegate: ConnectionDelegate):
         super().__init__(delegate=delegate)
-        self.__channel_pool = ChannelPool()
+        self.__channel_pool = self._create_channel_pool()
+
+    # noinspection PyMethodMayBeStatic
+    def _create_channel_pool(self):
+        return ChannelPool()
 
     def bind(self, address: Address = None, host: str = None, port: int = 0):
         if address is None:
+            assert port > 0, 'address error: (%s:%d)' % (host, port)
             address = (host, port)
         channel = self._get_channel(remote=None, local=address)
         if channel is None:
@@ -86,7 +132,7 @@ class PackageHub(BaseHub, ABC):
     # noinspection PyMethodMayBeStatic
     def _create_channel(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket) -> Channel:
         # override for user-customized channel
-        return PackageChannel(remote=remote, local=local, sock=sock)
+        return PacketChannel(remote=remote, local=local, sock=sock)
 
     # Override
     def _all_channels(self) -> Iterable[Channel]:
@@ -108,36 +154,27 @@ class PackageHub(BaseHub, ABC):
 
     # Override
     def open(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
-        if local is None:
-            assert remote is not None, 'both remote, local addresses empty!'
-            # get any channel
-            channels = self._all_channels()
-            for sock in channels:
-                address = sock.remote_address
-                if address is None or address == remote:
-                    return sock
-            # channel not found
-        else:
-            return self._get_channel(remote=remote, local=local)
+        # get channel with direction (remote, local)
+        return self._get_channel(remote=remote, local=local)
 
 
-class ServerHub(PackageHub):
-    """ Package Server Hub """
+class ServerHub(PacketHub):
+    """ Datagram Server Hub """
 
     # Override
     def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
-        gate = self.delegate
-        conn = BaseConnection(remote=remote, local=local, channel=channel, delegate=gate)
+        conn = BaseConnection(remote=remote, local=local, channel=channel)
+        conn.delegate = self.delegate  # gate
         conn.start()  # start FSM
         return conn
 
 
-class ClientHub(PackageHub):
-    """ Package Client Hub """
+class ClientHub(PacketHub):
+    """ Datagram Client Hub """
 
     # Override
     def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
-        gate = self.delegate
-        conn = ActiveConnection(remote=remote, local=None, channel=channel, delegate=gate, hub=self)
+        conn = ActiveConnection(remote=remote, local=None, channel=channel, hub=self)
+        conn.delegate = self.delegate  # gate
         conn.start()  # start FSM
         return conn
